# script to test the oddity surrounding COG or COM angular restraints using CustomCompoundBondForce in OpenMM

# import the OpenMM stuff
from simtk.openmm.app import *
from simtk.openmm import *
from simtk.unit import *

# import stdout so that we can toss data to the terminal
from sys import stdout
# import pi
from math import pi

# import our custom angle restraint module
import AngleRestraint

# define a little function for printing out our values of interest
def printStats():
	# get the actual angle
	state = simulation.context.getState(getPositions=True, getEnergy=True)
	currentPositions = state.getPositions()
	#print currentPositions
	actualAngle = AngleRestraint.CalculateAngle(currentPositions, angleG1, angleG2, angleG3, angleG1M, angleG2M, angleG3M, 'COG')
	# calculate the potential energy
	energy = state.getPotentialEnergy() / (kilocalorie / mole)
	# print stuff out
	print "Step", i, ", Restraint angle", restraintCenterDeg, ", Actual angle", actualAngle, ", Potential energy", energy


########### do the basic setup #####################
# set the default values for the simulation
testCase = 'water'

# load the files
if testCase == 'water':
	prmtop = AmberPrmtopFile('Water_Trimer_TIP3P.prmtop')
	pdb = PDBFile('Water_Trimer_TIP3P.pdb')
elif testCase == 'ions':
	prmtop = AmberPrmtopFile('Triple_Sodium.prmtop')
	pdb = PDBFile('Triple_Sodium.pdb')	
else:
	print 'could not understand test case, using the water trimer'
	testCase = 'water'
	prmtop = AmberPrmtopFile('Water_Trimer_TIP3P.prmtop')
	pdb = PDBFile('Water_Trimer_TIP3P.pdb')	
	
# create the system
system = prmtop.createSystem(constraints=HBonds, nonbondedMethod=NoCutoff)

############ setup the angular restraint ############
# set the atom indexes involved
if testCase == 'water':
	angleG1 = [0, 1, 2]
	angleG2 = [3, 4, 5]
	angleG3 = [6, 7, 8]
elif testCase == 'ions':
	angleG1 = [0]
	angleG2 = [1]
	angleG3 = [2]

# set the masses in case we do center of mass restraints
angleG1M = []
angleG2M = []
angleG3M = []
for i in range(system.getNumParticles()):
	if i in angleG1:
		angleG1M.append(system.getParticleMass(i))
	if i in angleG2:
		angleG2M.append(system.getParticleMass(i))
	if i in angleG3:
		angleG3M.append(system.getParticleMass(i))

# setup the restraint force
angleParticles = angleG1 + angleG2 + angleG3 
angleNumParticles = len(angleParticles)

restraintForceKcalMolDeg = 1.0
restraintForce = restraintForceKcalMolDeg * 4.184 * 57.2958**2
restraintCenterDeg = 180.0
restraintCenter = (restraintCenterDeg / 360.0) * (2.0 * pi)
restraintCenterType = 'COG'
restraintCalculationType = 'full'

angleRestraintString = AngleRestraint.CreateAngleRestraintString(angleG1, angleG2, angleG3, angleG1M, angleG2M, angleG3M, restraintCenterType, restraintCalculationType)

angleRestraintForce = CustomCompoundBondForce(angleNumParticles, angleRestraintString)
params = []
angleRestraintForce.addGlobalParameter('angleK', restraintForce)
angleRestraintForce.addGlobalParameter('theta0', restraintCenter)
CustomCompoundBondForce.addBond(angleRestraintForce, angleParticles, params)
system.addForce(angleRestraintForce)

print 'Successfully created an angle restraint at', restraintCenterDeg, 'degrees.  With forceConstant', restraintForce, 'kJ/mol*rad^2'
#print angleRestraintString

############# setup the system #################
# define some values
timeStep = 0.5 * femtosecond	# set to 0.001 in order to avoid NaN, but get odd behavior with rapid internal rotations of the waters
outputFrequency	= 10
totalTimeSteps = 1000
temperature = 100.0 * kelvin

# add the integrator
integrator = LangevinIntegrator(temperature, 1/picosecond, timeStep)

# finish setting up the simulation
simulation = Simulation(prmtop.topology, system, integrator)
simulation.context.setPositions(pdb.positions)

# energy minimize
simulation.minimizeEnergy()

# add a DCD reporter to we can see any odd stuff
simulation.reporters.append(DCDReporter('trajectory.dcd', outputFrequency))

# run the simulation in blocks, stopping to output angle and energy information, and changing the angle
print '\n'
"""
angles = [180.0, 170.0, 160.0, 150.0, 140.0, 130.0, 120.0, 110.0, 100.0, 90.0, 80.0, 70.0, 60.0, 50.0, 40.0, 30.0, 20.0, 10.0, 0.0]
for a in angles:
	restraintCenterDeg = a
	restraintCenter = (restraintCenterDeg / 360.0) * (2.0 * pi)
	simulation.context.setParameter("theta0", restraintCenter )
	for i in range(0, int(totalTimeSteps / outputFrequency)):
		printStats()
		simulation.step(outputFrequency)
printStats()
"""

for i in range(0, int(totalTimeSteps / outputFrequency)*10):
	printStats()
	simulation.step(outputFrequency)

